<?php
/* $Id: dbfm_admin.inc,v 1.1.2.31 2010/06/02 16:57:53 geoffeagles Exp $ */


/**
 * Implementation of hook_settings(). This is the main admin form
 */
function dbfm_admin_settings() {
  //add the JS code that allows us to hide and expose the attachments buttons. Code is triggered when the user changes the attach value 
  $path = drupal_get_path('module', 'dbfm');
//  drupal_add_js($path . '/js/admin.js');

  $form['dbfm_main'] = array (
    '#type' => 'fieldset',
    '#title' => 'Basic settings',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE, 
  );

  $form['dbfm_main']['dbfm_root_dir'] = array(
    '#type' => 'textfield',
    '#title' => t('Root directory'),
    '#default_value' => variable_get('dbfm_root_dir', 'dbfm'),
    '#maxlength' => '100',
    '#size' => '70',
    '#description' => t('Root directory used to present the filebrowser interface.  Users will not be able<br />to go up from this folder. NO SLASH')
  );

  $modulepath = drupal_get_path('module', 'dbfm');
  $form['dbfm_main']['dbfm_icon_dir'] = array(
    '#type' => 'textfield',
    '#title' => t('Icon directory'),
    '#default_value' => variable_get('dbfm_icon_dir', $modulepath . '/image/icon'),
    '#maxlength' => '100',
    '#size' => '70',
    '#description' => t('Name of directory where file type icons are stored (relative to base url).')
  );

  $form['dbfm_main']['dbfm_node_grp'] = array(
    '#type' => 'textfield',
    '#title' => t('Node group type'),
    '#default_value' => variable_get('dbfm_node_grp', 'groupnode'),
    '#maxlength' => '20',
    '#size' => '20',
    '#description' => t('If you are using group access permissions you will need to complete the node type given to group nodes'),
  );

  $form['dbfm_main']['dbfm_dbreset'] = array(
    '#type' => 'checkbox',
    '#title' => t('Reset filecounts'),
    '#default_value' => false,
    '#description' => t('If you suspect that the database filecounts may be wrong then check this (doesnt hurt anyway though it gobbles up db resource)')
  );

  $form['dbfm_main']['dbfm_nozip'] = array(
    '#type' => 'textarea',
    '#title' => t("Don't compress"),
    '#default_value' => variable_get('dbfm_nozip', 'arc arj as b64 bin btoa bz cab cpt gz hqx iso lha lzh mim mme pak pf rar sea sit sitx tar tbz tbz2 tgz uu uue z zip zoo'),
    '#description' => t('Files that are already compressed and should be stored as-is in the database (use space delimited with no leading dot)'),
  );

  //fill in which pages need to use dbfm
  $form['dbfm_main']['dbfm_pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Activate dbfm on specific page types'),
    '#default_value' => variable_get('dbfm_pages', "node edit,\nnode add"),
    '#description' => t('Enter one page type per line with a comma at the end of the line (for neatness). Example types might be "node" for all node pages or "node add" which limits the user to having greybox only when adding nodes. You shouldn\'t include the slashes. The reason you would want to disallow dbfm is because it saves loading the javascript code into the page header - pages should load faster! and it reduces possible interactions with other javascript'),
  );
           
  $form['dbfm_main']['dbfm_extensions'] = array(
    '#type' => 'textfield',
    '#title' => t('Excluded file extensions'),
    '#default_value' => variable_get('dbfm_extensions', 'bat exe'),
    '#maxlength' => 255,
    '#description' => t("Global list of extensions that users may NOT upload. (use space delimited with no leading dot). Note that these settings can be overridden on the basis of the user's role")
  );

  $form['dbfm_main']['dbfm_uploadsize'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum file size per upload'),
    '#default_value' => variable_get('dbfm_uploadsize', 1),
    '#size' => 5,
    '#maxlength' => 5,
    '#description' => t('The maximum size of a file a user can upload (in megabytes). Note that this setting may be overridden for individual roles'),
  );

  $form['dbfm_main']['dbfm_fnamesize'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum filename size'),
    '#default_value' => variable_get('dbfm_fnamesize', 40),
    '#size' => 5,
    '#maxlength' => 5,
    '#description' => t('The maximum number of characters in a filename. It is important that the maximum number is never reduced below the longest current filename on the system (note: directory names are fixed at 40 characters).'),
  );
            
  $form['dbfm_main']['dbfm_context'] = array(
    '#type' => 'textfield',
    '#title' => t('Point at which context menu flips'),
    '#default_value' => variable_get('dbfm_context', 0.70),
    '#size' => 5,
    '#maxlength' => 5,
    '#description' => t('The point at which the context menu flips  e.g if set to 0.75 the menu should display below the cursor until a point about three quarters down the screen, then above the cursor after that point'),
  );
  
  $form['dbfm_main']['dbfm_home_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Support home directories for users'),
    '#default_value' => variable_get('dbfm_home_enabled', false),
    '#description' => t('Whenever a new user is added a HOME directory will be added for them in dbFM.'),
  );

  $form['dbfm_main']['dbfm_home_dir'] = array(
    '#type' => 'textfield',
    '#title' => t('Home root directory'),
    '#default_value' => variable_get('dbfm_home_dir', 'HOME'),
    '#maxlength' => '50',
    '#size' => '50',
    '#description' => t("Root directory for users HOME areas, the directory will be created if it doesn't already exist, it will be created directly under the top level directory")
  );

  //checking this box will have no effect if the home_enabled checkbox hasn't been ticked
  $form['dbfm_main']['dbfm_homegen'] = array(
    '#type' => 'checkbox',
    '#title' => t('Generate home directories for users'),
    '#default_value' => false,
    '#description' => t('If you have just enabled home directories, or if you have recently changed user roles/permissions then check this to automatically generate home directories for existing users. NOTE that any existing home directories will be unaffected'),
  );
  
  //note that disabling the tree has NO significant effect on anything just now - the whole tree is already built on the client
  //however when the tree is rewritten pressing expand all will come at a much higher cost and should generally be discouraged when 
  //the filesystem gets larger
  $form['dbfm_main']['dbfm_exptree'] = array(
    '#type' => 'checkbox',
    '#title' => t('Expand Tree'),
    '#default_value' => true,
    '#description' => t("Enable or disable the 'Expand All' icon on the client's dbFM directory tree")
  );
  
  //we've given this an ID - dbfm-attach - to make it easier to address using JQuery         
  $form['dbfm_main']['dbfm_attach'] = array(
    '#prefix' => '<div id="dbfm-attach">',
	'#type' => 'radios',
    '#title' => t('Attachments'),
    '#default_value' => variable_get('dbfm_attach',0),
    '#options' => array(t('Attachments Off'),t('All roles allowed to add attachments'),t('Restrict Attachments to specific roles')),
    '#description' => t("Enable or disable dbfm file attachments to nodes"),
    '#suffix' => '</div>',
  );
            
  //an obvious additional load on the server
  $form['dbfm_main']['dbfm_refreshdelay'] = array(
    '#type' => 'textfield',
    '#title' => t('Re-query delay'),
    '#default_value' => variable_get('dbfm_refreshdelay', 0),
    '#maxlength' => '3',
    '#size' => '3',
    '#description' => t('The client will automatically re-query and refresh from the server every n minutes to find changes to the directory structure. A blank entry or value of 0 will switch off the client autorefresh')
  );

/*
disabled because it didn't work and was a fair ammount of work to get working
  $form['dbfm_main']['dbfm_usersize'] = array(
    '#type' => 'textfield',
    '#title' => t('Total file size per user'),
    '#default_value' => variable_get('dbfm_usersize', 10),
    '#size' => 5,
    '#maxlength' => 5,
    '#description' => t('The maximum size of all files a user can have on the site (in megabytes). Note that this setting may be overridden for individual roles')
  );
*/
  $form['dbfm_main']['dbfm_resetextn'] = array(
    '#type' => 'checkbox',
    '#title' => t('Reset Extensions and File Limits'),
    '#default_value' => false,
    '#description' => t('To reset excluded file extensions and file sizes for all roles (to the default values above) check this box'),
  );
  return system_settings_form($form);
}


/**
 * dbfm_subscription_settings
 * the admin screen has been split into several smaller screens. This screen groups together all the subscription settings
 */  
function dbfm_subscription_settings() {
   
  $form['dbfm_main'] = array (
    '#type' => 'fieldset',
    '#title' => 'Subscription settings',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE, 
  );

  //allow the option of switching subscriptions off - that should be the default. Subscriptions significantly increase server/database load and slow the building of
  //the browser page
  $form['dbfm_main']['dbfm_subson'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow Subscriptions'),
    '#default_value' => variable_get('dbfm_subson', false),
    '#description' => t('Switches on subscriptions - if this box is checked then the user may subscribe to a document (to be notified of any changes to that document)'),
  );
 
   //number of items per page for the my documents and subscriptions pages   
  $form['dbfm_main']['dbfm_subppage'] = array(
    '#type' => 'textfield',
    '#title' => t('Items per page'),
    '#default_value' => variable_get('dbfm_subppage', 20),
    '#maxlength' => 10,
    '#description' => t('The maximum number of files per page on the "My Documents" and "My Subscriptions" pages'),
  );

  //prompt for the address from whoom the subscription emails should be sent   
  $form['dbfm_main']['dbfm_subscribemail'] = array(
    '#type' => 'textfield',
    '#title' => t('email for sending subscription notifications'),
    '#default_value' => variable_get('dbfm_subscribemail', '"DO NOT REPLY" <sendmail.test@eservglobal.com>'),
    '#maxlength' => 100,
    '#description' => t('The "from" address for subscription / notification emails'),
  );

  $form['dbfm_main']['dbfm_change_subj'] = array(
    '#type' => 'textfield',
    '#title' => t('Change email - Subject Line'),
    '#default_value' => variable_get('dbfm_change_subj', 'File details change - file details (metadata or permissions) for [fname] ([fid]) have been edited'),
    '#maxlength' => '100',
    '#size' => '80',
    '#description' => t('Enter the subject line for the email that should be sent to subscribers when file metadata or permissions are changed. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the change in the format "[name]". NOTE: You may not use the newline character in the subject')
  );
      
  $form['dbfm_main']['dbfm_change_body'] = array(
    '#type' => 'textarea',
    '#title' => t("Change email - Content"),
    '#default_value' => variable_get('dbfm_change_body', 'The file [fpath] has been changed (either the metadata or permissions have been edited) by user [name]. Should you have any queries the email address of the user making the changes is [mail]'),
    '#description' => t('Enter the contents of the email message that should be sent to subscribers when file metadata is changed. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the change in the format "[name]"'),
  );

  $form['dbfm_main']['dbfm_revise_subj'] = array(
    '#type' => 'textfield',
    '#title' => t('New Revision email - Subject Line'),
    '#default_value' => variable_get('dbfm_revise_subj', 'New Revision - A new revision of [fname] ([fid]) has been added'),
    '#maxlength' => '100',
    '#size' => '80',
    '#description' => t('Enter the subject line for the email that should be sent to subscribers when a new file revision has been added. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the addition in the format "[name]". NOTE: You may not use the newline character in the subject'),
  );
      
  $form['dbfm_main']['dbfm_revise_body'] = array(
    '#type' => 'textarea',
    '#title' => t("New Revision email - Content"),
    '#default_value' => variable_get('dbfm_revise_body', 'A new revision - [fpath] - has been added by user [name]. Should you have any queries the email address of the user adding the revision is [mail]'),
    '#description' => t('Enter the contents of the email message that should be sent to subscribers when a new file revision is added. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the change in the format "[name]"'),
  );

  $form['dbfm_main']['dbfm_convert_subj'] = array(
    '#type' => 'textfield',
    '#title' => t('Convert to version email - Subject Line'),
    '#default_value' => variable_get('dbfm_convert_subj', 'Convert to version - [fname] ([fid]) has been converted to a version directory'),
    '#maxlength' => '100',
    '#size' => '80',
    '#description' => t('Enter the subject line for the email that should be sent to subscribers when a file is converted to a version directory. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the addition in the format "[name]". NOTE: You may not use the newline character in the subject'),
  );
      
  $form['dbfm_main']['dbfm_convert_body'] = array(
    '#type' => 'textarea',
    '#title' => t("Convert to version email - Content"),
    '#default_value' => variable_get('dbfm_convert_body', '[fname] - has been converted to a version directory by user: [name]. Should you have any queries the email address of the user performing the conversion is [mail]'),
    '#description' => t('Enter the contents of the email message that should be sent to subscribers when a file is converted to a version directory. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has made the change in the format "[name]"'),
  );

  $form['dbfm_main']['dbfm_overwrite_subj'] = array(
    '#type' => 'textfield',
    '#title' => t('Overwrite email - Subject Line'),
    '#default_value' => variable_get('dbfm_overwrite_subj', 'Overwrite - [fname] ([fid]) has been overwritten'),
    '#maxlength' => '100',
    '#size' => '80',
    '#description' => t('Enter the subject line for the email that should be sent to subscribers when a file is overwritten. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who overwrote the file in the format "[name]". NOTE: You may not use the newline character in the subject'),
  );
 
  $form['dbfm_main']['dbfm_overwrite_body'] = array(
    '#type' => 'textarea',
    '#title' => t("Overwrite email - Content"),
    '#default_value' => variable_get('dbfm_overwrite_body', 'The file [fpath] has been overwritten by user [name]. Should you have any queries the email address of the user overwriting the file is [mail]'),
    '#description' => t('Enter the contents of the email message that should be sent to subscribers when a file is overwritten. Your message may include document database fields using the format "[fname]". They can also include information pertaining to the user who overwrote the file in the format "[name]"'),
  );

  $form['dbfm_main']['dbfm_delete_subj'] = array(
    '#type' => 'textfield',
    '#title' => t('Delete email - Subject Line'),
    '#default_value' => variable_get('dbfm_delete_subj', 'Deletion - [fname] ([fid]) has been deleted'),
    '#maxlength' => '100',
    '#size' => '80',
    '#description' => t('Enter the subject line for the email that should be sent to subscribers when a file is deleted. Your message may include document database fields using the format "[fname]". It can also include information pertaining to the user who has deleted the file in the format "[name]". NOTE: You may not use the newline character in the subject'),
  );

  $form['dbfm_main']['dbfm_delete_body'] = array(
    '#type' => 'textarea',
    '#title' => t("Delete email - Content"),
    '#default_value' => variable_get('dbfm_delete_body', 'The file [fpath] has been deleted by user [name]. Should you have any queries the email address of the user making the deletion is [mail]'),
    '#description' => t('Enter the contents of the email message that should be sent to subscribers when a file is deleted. Your message may include document database fields using the format "[fname]". They can also include information pertaining to the user who has deleted the file in the format "[name]"'),
  );

  return system_settings_form($form);
}

/**
 * dbfm_role_settings
 * the admin screen has been split into several smaller screens. This screen groups together all the role settings
 */  
function dbfm_role_settings() {
   
  $form['dbfm_main'] = array (
    '#type' => 'fieldset',
    '#title' => 'Role settings',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE, 
  );

  // returns an array of valid roles (avoids anonymous users)
  $roles = user_roles(1,0);

  // Flush extensions regex cache for upload and db enum 
  dbfm_get_extensions_regex(FLUSH);

  foreach ($roles as $rid => $role) {
    $form['dbfm_main']["settings_role_".$rid] = array(
      '#type' => 'fieldset',
      '#title' => t('Settings for @role', array('@role' => $role)),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );

    $form['dbfm_main']["settings_role_".$rid]["dbfm_extensions_".$rid] = array(
      '#type' => 'textfield',
      '#title' => t('Exceptions to excluded file extensions'),
      '#default_value' => variable_get("dbfm_extensions_".$rid, ""),
      '#maxlength' => 255,
      '#description' => t('To add additional file extensions to the global exclusion list enter a plus before the extension. To remove a file extension from the exclusion list enter a minus before the extension (use space delimited with no leading dot). e.g "+zip +tar -exe". Applies only to this role. NOTE that a blank entry means the default values will be used'),
    );

    $form['dbfm_main']["settings_role_".$rid]["dbfm_uploadsize_".$rid] = array(
      '#type' => 'textfield',
      '#title' => t('Maximum filesize per upload'),
      '#default_value' => variable_get("dbfm_uploadsize_".$rid, ""),
      '#size'=> 5,
      '#maxlength' => 5,
      '#description' => t('To override the default value for maximum upload size enter a value here (in megabytes). Applies only to this role. NOTE that a blank entry means the default value will be used'),
    );

    //if the admin user has chosen to only allow certain roles to access attachments then show this box (values for attach are 0 (none), 1 (all users), 
	//2 restricted users
    $form['dbfm_main']["settings_role_".$rid]["dbfm_attach_".$rid] =  array(
      '#type' => 'checkbox',
      '#prefix' => '<div class="dbfm-att">',
      '#title' => t('Enable node attachments'),
      '#default_value' => variable_get("dbfm_attach_".$rid, false),
      '#description' => t('Check box to enable node attachments for this role'),
      '#suffix' => '</div>',
    );
  } //end of foreach
  return system_settings_form($form);
}


/**
 * Submit code for the admin forms
 * @param int $form_id
 * @param array $form_values
 */

function dbfm_admin_settings_submit($form_id, $form_values) {
  global $user;
  $err = "";  //declare a string used to return error messages

  //we need a little extra code on the admin form submit to make sure that the root directory is saved to the database
  //test whether they've set BLOB to true and make an entry in the database to hold the root directory name
  //if this is ever changed all the files will automatically move with it (because they're parented by it) BUT
  //their paths will need to be edited - if the system is large this is a very bad idea, get the name right first!

  //we should also be creating the root directory, if it doesn't already exist, for non-blob systems
  // we can easily spot whether they're changing an existing entry!!!

  $setmime = 'directory';

  //we need to test whether there's a root entry in the database - don't need to do it like this - the root entry will always have
  //an fid of 1
  //the third parameter suppresses errors
  $rootrec = dbfm_selected_file_record(1, '', $err);
  if ($rootrec == FALSE) {
    // there's no root entry in the database, we'll have to add one, do some checking on the name to
    //ensure it's valid
    if (checkname($form_values['dbfm_root_dir'])) {
      // insert the new entry
      $dirpath = '/' . trim($form_values['dbfm_root_dir']);
      $currentdate = date("Y-m-d H:i:s");

      $result = db_query("INSERT INTO {dbfm_file} (fid, uid, fpath, fname, fsize, fmime, fcreatedate, flastmod, fparent, fwaccess) VALUES (%d, %d, '%s', '%s', %d, '%s', '%s', '%s', %d, %d)",1, $user->uid, $dirpath, trim($form_values['dbfm_root_dir']), 0, $setmime, $currentdate, $currentdate, 0, READ_WRITE);
      // FOOBAR1
      $result = db_query("INSERT INTO {dbfm_perm} (fid, rgaccess, rgid) VALUES (%d, %d, %d)", 1, 0, 0);
      if ($result === FALSE) {
        drupal_set_message("ERROR: Failed to insert the directory " . $dest . " into the database");
        return FALSE;
      }
    }
    else {
      // the checkname function will have returned an appropriate error message
      return FALSE;
    }
  }
  else {
    //we HAVE already got a root directory entry in the database

    //found a root entry, are they trying to change it? if so do an update
    if ($form_values['dbfm_root_dir'] != $rootrec->fname) {
      if (checkname($form_values['dbfm_root_dir']) == FALSE) {
        //should have reported its error
        return FALSE;
      }

      $dirpath = '/' . trim($form_values['dbfm_root_dir']);
      $currentdate = date("Y-m-d H:i:s");

      $result = db_query("UPDATE {dbfm_file} SET fpath = '%s', fname = '%s', flastmod = '%s' WHERE fid = %d;", $dirpath, trim($form_values['dbfm_root_dir']), $currentdate, $rootrec->fid);

      if ($result === FALSE) {
        drupal_set_message("ERROR: Failed update the root directory to " . $form_values['dbfm_root_dir']);
        return FALSE;
      }

      if ($rootrec->fsize > 0) {
        //now we need to recursively rename everything underneath
        $ret = TRUE;
        $err_arr = "";

        // we need to update the local rootrec before calling a recursive routine to modify all the child records
        $rootrec->fpath = $dirpath;

        $ret = dbfm_recur_dbrename($rootrec, $ret, $err_arr);
        if ($ret == FALSE) {
          drupal_set_message($err_arr);
          return $ret;
        }
      }
    }
  }

  //calls a function to rebuild the directory size values in the database - could take a while...
  if ($form_values['dbfm_dbreset']) {
    dbfm_resetdb();
  }
  
  //the called function will still see the old name for the home root directory - we need to pass it
  if ($form_values['dbfm_homegen']) {
    dbfm_homegenerate($form_values['dbfm_home_dir']);
  }

  //how do we get it to execute the default code as well? - here's how!
  system_settings_form_submit($form_id, $form_values);

  //calls a function to reset the default file sizes for each user role
  if ($form_values['dbfm_resetextn']) {
    $roles = user_roles(1,0);
    foreach ($roles as $rid => $role) {
      variable_set('dbfm_extensions_'.$rid, '');
      variable_set('dbfm_uploadsize_'.$rid, '');
      variable_set('dbfm_usersize_'.$rid, '');
    }
  }
}

/**
 * this function generates user home directories for all the existing users
 * if the HOME root directory doesn't already exist it will be created
 * @param homedirname - using the varable for the home root directory was unsuccessful - at this point it has not been saved to the database - so it needs to be
 * explicitly passed
 */ 
function dbfm_homegenerate($homedirname) {

  //run through the user table creating entries for every user - provided their roles allow for a home directory 
  $userlist = db_query("SELECT uid, name FROM {users}" );
  
  while ($userrec = db_fetch_object($userlist)) {
    //we have to test first whether the user is supposed to have a home directory - if not we continue without creating one
    if (check_home($userrec->uid)) {  
      //call a function to create the user's home directory - takes parameters username, uid and the name of the home root directory
      $dirfid = create_userhome($userrec->name, $userrec->uid, $homedirname);
      //if we failed to create a home directory an error message should have been created in the log
      if ($dirfid != FALSE) {   
        //now we need to make sure the user's dbfm_home entry is updated to record the home directory fid - any existing entry will be overwriten
        $user = user_load(array('uid' => $userrec->uid));
        $homearr = array('dbfm_home' => $dirfid);
        user_save($user, $homearr);
      } 
	}   
  }
}
