<?php
/* $Id: dbfm_cronstuff.inc,v 1.1.2.25 2010/02/12 16:46:20 geoffeagles Exp $ */

//DON'T FORGET - when you're testing cron that changing the variable entry for dbfm_lastreplicate in the database won't necessarily work since the variables 
//table is held in cache

//Developers should note that there is NO login protection for cron - ANYONE can run it if they can guess the URL ... that's hard isn't it!
//If cron firing off a lot of work then someone repeatedly running it could compromise your server. Might be worth adding protection within the code
//to protect from this???

//a quick note about crontype - originally I planned to deal with search for dbFM within this cron code. Different jobs would have a different crontype: 
//search was 1 and replication 2.
//the crontype is now largely redundant since search has its own way of doing things

/**
 *
 * dbfm_cron_shutdown - called by the cron hook. This does the actual cron work - hopefully without being limited by the 30 second 
 * cron timout which I'm told exists on our servers (and which may be as low as 10 seconds on others!)
 * ALL errors and messages will go into the local watchdog table
 * 
 */
function dbfm_cron_shutdown() {
  //do all the useful stuff here

  //test whether we are a slave - only slaves need to do replication, also test whether replication's allowed to run
  if (variable_get('replication_server_mode','0') != 2) {
    return;
  }
  
  if (variable_get('replication_online','0') == FALSE) {
    watchdog('dbfm', t('Master offline - Replication process switched off'));
    return;  //can't do anything if we're not connected to the master
  }

  //new admin option to allow cron processing to be disabled
  if (variable_get('dbfm_cron_repenabled','0') == 0) {
    watchdog('dbfm', t('Replication process switched off'));
    return;           //cron switched off by admin
  }
    
  set_time_limit(0);  //switches off the default php timeout, if we don't do this we'll probably have the rug pulled after 30 secs
    
  // Fetch the flag indicating whether a previous process is already running - we can't use variable_get because of problems with caching
  $varrecs = db_query("SELECT * FROM {variable} WHERE name = '%s'", "dbfmrep_semaphore");
  $numret = db_num_rows($varrecs);

  if ($numret == 0) {
    $semaphore = FALSE;  //OK, there's no previous semaphore
  }
  else {
    $semaphore = $conf["dbfmrep_semaphore"]; //the variable table entries are cached in a global array - $conf
  }
   
  // if there was a previous semaphore is it valid? The semaphore is an integer containing the time it was initially set
  if ($semaphore) {
    if ((time() - $semaphore) > 10800) {
      // the previous process has been running for 3 hours - unlikely! We'll delete the semaphore and start again
      watchdog('dbfm', t('Warning: Replication seems to have been running now for 3 hours - this is abnormal. The replication flag will be deleted and file replication will restart at the next cron run.'), WATCHDOG_WARNING);

      // Release the semaphore and quit until the next run
      variable_del('dbfmrep_semaphore');
    }
    else {
      // replication is still running.
      watchdog('dbfm', t('Attempting to re-run file replication while it is already running.'), WATCHDOG_WARNING);
    }
    return false;
  }
  
  watchdog('dbfm', t('Replication process started.' . date("Y-m-d H:i:s")));
  // store the current time in the semaphore
  variable_set('dbfmrep_semaphore', time());

  //make a todo list - a list of all those records in dbfm which need replicating add entries to dbfm_cronlist
  dbfm_todolist();
  //no point in checking a return value since we still need to process any cronlist entries that are already there
    
  //replicate all the outstanding dbfm_data files (and update version records)
  $res = dbfm_repfiles();
    
  //the semaphore is released 
  variable_del('dbfmrep_semaphore');

  if ($res > -1) {
    watchdog('dbfm','Repfiles processed ' . $res . ' records');
    watchdog('dbfm', t('dbfm replication process completed.' . date("Y-m-d H:i:s")));
  }
  return;
}

/**
 *
 * dbfm_todolist - called by the cron hook. This function retrieves a list of dbfm_file records which have changed since the last 
 * cron run. Appends the list to the table dbfm_cronlist 
 * reports any errors to the watchdog
 * @return The function doesn't return any value as such - even if there are no records added to the cronlist the calling program will still need to 
 * check the cronlist because it may have to do retries on existing entries
 */

function dbfm_todolist() {
  //updates to the replication index
  $lastrun = variable_get('dbfm_lastreplicate', date("Y-m-d H:i:s"));
    
  //replace the variable with the current date (a problem if the selects that follow screw up!)
  //variable_set('dbfm_lastreplicate',date("Y-m-d H:i:s"));
  
  $query = "SELECT fid, fblobmod, fmime, fpath, fmd5, fparent, fsize, fversion FROM {dbfm_file} WHERE fblobmod > '" . $lastrun . "' AND fparent > -1 ORDER BY fblobmod DESC";
  //I'm assuming there will be no issues here with date/time and different timezones - everything should be GMT (or is it CET??)

  //do a select statements to return all the records from dbfm_file whos flastmod or fblobmod occured after the last time 
  //this function ran.
  if (($result = db_query($query)) == FALSE) {
    watchdog('dbfm', t('dbfm cron - Problem completing the todo list for file replication'));
    return;
  }
  
  $numret = db_num_rows($result);
  if ($numret == 0) {
    //nothing to process
    watchdog('dbfm', t('No new records added to replication list'));   
    return;
  }
  $newentries = 0;
  while ($frec = db_fetch_object($result)) {
    //replication entries
    if (($frec->fmime != 'directory') AND ($frec->fmime != 'version')) {
      $insquery = sprintf("INSERT INTO {dbfm_cronlist} (fid, fpath, crontype, fmd5, fdate, fparent, fmime, fsize, fversion, retries, status) VALUES ( '%d', '%s', '%d', '%s', '%s', '%d', '%s', '%d', '%d', '%d', '%s')", $frec->fid, $frec->fpath, 2, $frec->fmd5, $frec->fblobmod, $frec->fparent, $frec->fmime, $frec->fsize, $frec->fversion, 0, "new");
      db_query($insquery);  //not checking the return value here
      $newentries = $newentries + 1;
    }
  }  //end of while loop
  //report back how many new entries need replcating
  watchdog("dbfm", $newentries . " records added to the list awaiting replication");
  return;
}


/**
 * dbfm_repfiles -called as part of the cron hook for dbfm
 * replicates any file changes to dbfm on the slave since the last time the function was called. Uses the todo list (dbfm_cronlist)
 * progress/status and most problems encountered will be noted on the todo list - all serious problems will be logged to
 * appear in the watchdog. 
 * @return returns the number of data files replicated or -1 if something's gone wrong
 */
function dbfm_repfiles() {

  //should we leave entries in the todo list if they've reached their max retry value - Yup
  $repcount = 0;  //counts the actual number of files dealt with (replicated files plus files that didn't need replicating 
                  //and were deleted
  $isnew = true;  //flag to indicate whether we need to add a new record to dbfm_data (true) or update an existing record (false)

  $time_start = acutime();
  $time_end = $time_start + variable_get("dbfm_cron_reptime",'2');  //the time in seconds that replication can run for

  //find the maximum number of retries for any single file
  $maxretries = variable_get("replication_http_retries",'5');

  //loop through the todo list - through the replication records that haven't yet exceeded their retry count
  //specifically we check any entries who's mime type is version
  //having found that entry we need to find another entry with the same fblobmod date. 
  //NOW michael argues that there's a possibility that there may be other fblobmod's created at EXACTLY the same time down 
  //to the second ... so to please him we can check that the parent of the matching record is the version record AND since 
  //there could be two version files saved at the same time we need to check that we do the swop with version 1 of the file
  $query = "SELECT * FROM {dbfm_cronlist} WHERE crontype = 2 AND retries < " . $maxretries;
  $res = db_query($query);
  if ($res == FALSE) {
    watchdog('dbfm', t('dbfm_repfile: Unable to read from cronlist -') . mysql_error() . '-');
    return -1;
  }
  $numret = db_num_rows($res);
  if ($numret == 0) {
    //nothing to do
    watchdog('dbfm', t('cron repfile: no files to replicate in cron list'));
    return $repcount;
  }

  watchdog('dbfm', $numret . ' records awaiting replcation');
  //start of while loop
  //we've got some records to process
  while ($crec = db_fetch_object($res)) {
    //if we've run out of time then return. The allotted time is a little misleading - the lime taken to build the cronlist doesn't count towards the alloted time, 
	//ONLY the time taken processing the cronlist 
    if (acutime() > $time_end) {
      watchdog('dbfm', 'WARNING: replication exceded the permitted time');
      return $repcount;
    }
    
    //if there are a lot of records to replicate we should give periodic feedback in the watchdog as to progress. We'll update the log every 10 records
    if (($repcount % 10) == 0) {
      watchdog("dbfm", $repcount . " file records replicated in " .  (acutime() - $time_start) . " seconds", WATCHDOG_NOTICE);
    }

    $isnew = TRUE; //start by assuming every record will be a new one 
	 
    //first check whether a local dbfm_data record already exists and is correct - note that ALL actions should take place on
    //the local table
    $dquery = "SELECT fmd5, fblobmod FROM {dbfm_data} WHERE fid = " . $crec->fid;
    $dres = db_query($dquery);
    if ($dres == FALSE) {
      watchdog('dbfm', t('dbfm_repfile: Unable to query the dbfm_data table -') . mysql_error() . '-');
      //don't change the retry count - it's a problem accessing the table not with the data file itself
      continue;  //can't do much
    }
    $numret = db_num_rows($dres);

    //a record already exists in dbfm_data - the index on dbfm_data will prevent duplicates for a fid
    if ($numret == 1) {
      $drec = db_fetch_object($dres);
      //the md5 values match - there's no difference in the file so don't bother doing anything
      if ($drec->fmd5 == $crec->fmd5) {
        //record's already been updated so DELETE the entry in the cronlist - specify date as well in case there are
        //further updates in the list
        $query = sprintf("DELETE FROM {dbfm_cronlist} WHERE fid = %d AND fdate = '%s'", $crec->fid, $crec->fdate);
        $resd = db_query($query);
        if ($resd == FALSE) {
          watchdog('dbfm','Unable to delete record from cronlist ' . $query . ' *1* ' . mysql_error() . '-');
        }
        else {
          $repcount++;
        }
        //dealt with - onto the next record
        continue;
      }
      else {
        //perhaps the records don't match because we have a later update?
        if ($drec->fblobmod != $crec->fdate) {
          //we already have a later version so ignore this update
          $query = sprintf("DELETE FROM {dbfm_cronlist} WHERE fid = %d AND fdate = '%s'", $crec->fid, $crec->fdate);
          $resd = db_query($query);
          if ($resd == FALSE) {
            watchdog('dbfm','Unable to delete record from cronlist ' . $query . ' *2* ' . mysql_error() . '-');
          }
          else {
            $repcount++;
          }
          continue;
        }
      } 

      //the remaining option not covered is that a file already exists in dbfm_data but the hash values don't match - we'll need to update 
      //the file
      $isnew = FALSE;
    }  //end of code to deal with a file already existing

    //if the file is the first version in a version directory then the data record MAY already be on the local server but still 
    //referencing the fid of the parent version directory. In this case we can update the fid on the data record
    if ($crec->fversion == 1) {
      //lets see if the data file's already here but assigned to the parent version directory
      $fidhash = $crec->fparent . "|" . $crec->fmd5;
      $frec = dbfm_file_data($fidhash, $err, $foundrec);

      if ($foundrec == 3) {
        //found it so lets do a local update 
        $query = sprintf("UPDATE {dbfm_data} SET fid = %d WHERE fid = %d", $crec->fid , $crec->fparent);
        $resu = db_query($query);
        if ($resu == FALSE) {
          watchdog('dbfm','Unable to update record in dbfm_data' . $query . ' *3* ' . mysql_error() . '-');
          //we can't delete the record in the todo list but we must update the retry count
          //update the todo list incrementing the retry count
          $query = sprintf("UPDATE {dbfm_cronlist} SET retries = %d WHERE fid = %d", ($crec->retries + 1), $crec->fid);
          $resu = db_query($query);
          if ($resu == FALSE) {
            watchdog('dbfm','Unable to update the cronlist retry count-' . $query . ' *4* ' . mysql_error() . '-');
          }
        } //end of code dealing with NOT being able to update the existing record
        else {
          //successfully updated the fid for the data record, delete the entry in the todo list (cronlist)
          $query = sprintf("DELETE FROM {dbfm_cronlist} WHERE fid = %d AND fdate = '%s'", $crec->fid, $crec->fdate);
          $resd = db_query($query);
          if ($resd == FALSE) {
            watchdog('dbfm','Unable to delete record from cronlist ' . $query . ' *5* ' . mysql_error() . '-');
          }
          else {
            $repcount++;
          }
        }
        continue;  //we've now dealt the case where we find a version which already had a local record
      }
      //if the version still needs to download the file from the server that will be dealt with below
    }  //end of code specific to versions

    //at this point we need to either update an existing record or insert a new record using data from the server
    //the parameter we use for that request is a concatonation of the fid a pipe ('|') and the md5 hash value
    $fidhash = $crec->fid . "|" . $crec->fmd5;

    //request the file from the master database
    $resrq = dbfm_rqfile_http($fidhash, $isnew, $crec->fsize);

    //if the call was successful then delete the cronlist record
    if ($resrq) {
      $query = sprintf("DELETE FROM {dbfm_cronlist} WHERE fid = %d AND fdate = '%s'", $crec->fid, $crec->fdate);
      $resd = db_query($query);
      if ($resd == FALSE) {
        watchdog('dbfm','Unable to delete record from cronlist ' . $query . ' *5* ' . mysql_error() . '-');
      }
      else {
        $repcount++;
      }
      continue;
    }
    else {
      //the rqfile function should give an appropriate watchdog message BUT lets have another one
      watchdog('dbfm','Unable to transfer the data file for fid: ' . $crec->fid);
      //so we don't increment the overall repcount but we do increment the retry count for the file
    }

    //if the call wasn't successful then update the todo list entry incrementing the retry count, note this query doesn't allow for entries with the same
    //fid which COULD conceivably happen - if there were 2 entries with the same fid their retries would go up at twice the rate!
    $query = sprintf("UPDATE {dbfm_cronlist} SET retries = %d WHERE fid = %d", ($crec->retries + 1), $crec->fid);
    $resu = db_query($query);
    if ($resu == FALSE) {
      watchdog('dbfm','Unable to update cronlist entry' . $query . ' *6* ' . mysql_error() . '-');
    }
  }  //end of while for if we have records to process
  $timetaken = acutime() - $time_start;
  watchdog('dbfm', 'Replication took ' . (float)$timetaken . ' seconds');
  return $repcount;
}


//function to return the time - uses the microtime function to give greater accuracy than time would otherwise give
//the return value is a float which details the time since the epoch in seconds and microseconds
function acutime() {
  list($usec, $sec) = explode(" ", microtime());
  return ((float)$usec + (float)$sec);
}
